package cn.com.qmth.examcloud.service.core.service;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.contains;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.transaction.Transactional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import cn.com.qmth.examcloud.common.uac.AccessCtrlUtil;
import cn.com.qmth.examcloud.common.util.ErrorMsg;
import cn.com.qmth.examcloud.service.core.dto.UserInfo;
import cn.com.qmth.examcloud.service.core.entity.Org;
import cn.com.qmth.examcloud.service.core.entity.Student;
import cn.com.qmth.examcloud.service.core.entity.User;
import cn.com.qmth.examcloud.service.core.entity.UserRole;
import cn.com.qmth.examcloud.service.core.enums.LoginType;
import cn.com.qmth.examcloud.service.core.enums.UserScope;
import cn.com.qmth.examcloud.service.core.enums.UserType;
import cn.com.qmth.examcloud.service.core.repo.OrgRepo;
import cn.com.qmth.examcloud.service.core.repo.StudentRepo;
import cn.com.qmth.examcloud.service.core.repo.UserRepo;
import cn.com.qmth.examcloud.service.core.service.sync.DataSendService;

/**
 * 学生服务类
 * Created by songyue on 17/1/14.
 */
@Service
public class StudentService {

    @Autowired
    StudentRepo studentRepo;

    @Autowired
    UserRepo userRepo;

    @Autowired
    UserService userService;

    @Autowired
    OrgRepo orgRepo;

    @Autowired
    DataSendService dataSendService;

    private static final String JPG = ".jpg";

    /**
     * 获取所有学生(分页)
     *
     * @param studentCriteria
     * @param pageable
     * @return
     */
    public Page<Student> getAllStudent(Student studentCriteria, Pageable pageable) {
        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                .withMatcher("name", contains())
                .withMatcher("studentCode", contains())
                .withMatcher("identityNumber", contains()).withIgnoreNullValues();
        Example<Student> studentExample = Example.of(studentCriteria, exampleMatcher);
        return studentRepo.findAll(studentExample, pageable);
    }

    /**
     * 获取所有学生
     *
     * @param studentCriteria
     * @return
     */
    public List<Student> getAllStudent(Student studentCriteria) {
        ExampleMatcher exampleMatcher = ExampleMatcher.matching();
        Example<Student> studentExample = Example.of(studentCriteria, exampleMatcher);
        return studentRepo.findAll(studentExample);
    }

    /**
     * 绑定学生照片
     *
     * @return
     */
    public List<ErrorMsg> photopBundled(Long rootOrgId, File directory) {
        List<ErrorMsg> list = new ArrayList<ErrorMsg>();
        if (directory.isDirectory()) {
            for (File file : directory.listFiles()) {
                if (!file.getName().endsWith(JPG)) {
                    list.add(new ErrorMsg(file.getName() + "格式错误，文件名格式必须是jpg(小写)"));
                }
                String fileName = file.getName().split(JPG)[0];
                //先查学号
                Student student = studentRepo.findByStudentCodeAndRootOrgId(fileName, rootOrgId);
                if (student == null) {//学号不存在查身份证号
                    student = studentRepo.findByIdentityNumber(fileName);
                }
                if (student != null) {//学生存在则绑定照片
                    student.setPhotoPath(student.getUser().getRootOrgId() + File.separator + "photo" + File.separator + fileName);
                    studentRepo.save(student);
                    dataSendService.sendStudent(student);
                }
            }
        } else {
            list.add(new ErrorMsg("文件路径不正确"));
        }
        return list;
    }

    @Transactional
    public Student save(Student student) {
        if (student.getUser() == null || null == student.getUser().getId()) {//判断是否有用户
            //判断是否有该学生,
            if (!StringUtils.isEmpty(student.getStudentCode())) {
                Student domain = studentRepo.findByRootOrgIdAndStudentCode(student.getUser().getRootOrgId(), student.getStudentCode());
                if (domain != null) {//学号查找不为空,更新身份证号
                    domain.setIdentityNumber(student.getIdentityNumber());
                    domain.setUpdateTime(new Date());
                    domain.setName(student.getName());
                    domain.setOrgId(student.getOrgId());
                    domain = studentRepo.save(domain);
                    dataSendService.sendStudent(domain);
                    return domain;
                }
            }

            Student entity = studentRepo.findByIdentityNumberAndRootOrgId(student.getIdentityNumber(), student.getRootOrgId());
            if (entity != null) {//身份证查找不为空，更新学号
                entity.setStudentCode(student.getStudentCode());
                entity.setUpdateTime(new Date());
                entity.setName(student.getName());
                entity.setOrgId(student.getOrgId());
                entity = studentRepo.save(entity);
                dataSendService.sendStudent(entity);
                return entity;
            }
            student.setOrgId(student.getUser().getOrgId());
            student.setRootOrgId(student.getUser().getRootOrgId());
            //新建用户和学生
            User user = new User(student.getName(), UserScope.ORG, student.getUser().getRootOrgId(), student.getUser().getOrgId(), UserType.STUDENT);
            user.setEnable(student.getUser().getEnable() == null ? true : student.getUser().getEnable());
            String password = null;
            if (!StringUtils.isEmpty(student.getIdentityNumber())) {//身份证号后6位
                if (!StringUtils.isEmpty(student.getStudentCode())) {
                    user.setLoginName(student.getStudentCode());
                } else {
                    user.setLoginName(student.getIdentityNumber());
                }
                password = student.getIdentityNumber().substring(student.getIdentityNumber().length() - 6, student.getIdentityNumber().length());
            }
            user.setPassword(password);
            List<UserRole> userRoles = new ArrayList<UserRole>();
            userRoles.add(new UserRole("ecs_oe", "STUDENT"));
            user.setUserRoles(userRoles);
            userRepo.save(user);
            student.setUser(user);
        }
        student = studentRepo.save(student);
        dataSendService.sendStudent(student);
        return student;
    }

    /**
     * 二级登录
     *
     * @param orgId
     * @param loginName
     * @param password
     * @return
     */
    public UserInfo login(String orgId,
                          String loginName,
                          String password,
                          LoginType loginType) throws Exception {
        Org org = orgRepo.findFirstByParentIdAndCode((long) 0, orgId);
        if (org == null) {
            throw new RuntimeException("学校不存在");
        }
        Student student = null;
        if (LoginType.STUDENT_CODE.equals(loginType)) {
            student = studentRepo.findByStudentCodeAndRootOrgId(loginName, org.getId());
        }else if (LoginType.IDENTITY_NUMBER.equals(loginType)) {
            student = studentRepo.findByIdentityNumberAndRootOrgId(loginName, org.getId());
        }
        if (student == null) {
            throw new RuntimeException("该用户不存在");
        }
        return this.loginProcess(student, password);
    }

    private UserInfo loginProcess(Student student, String password) throws Exception {
        if (student.getUser() == null) {
            throw new RuntimeException("该用户不存在");
        } else if (!student.getUser().getPassword().equals(password)) {
            throw new RuntimeException("密码错误");
        } else if (!student.getUser().getEnable()) {
            throw new RuntimeException("该用户被禁用");
        } else {
            String token = AccessCtrlUtil.buildToken();
            userService.initUserLogin(student.getUser());
            userService.createAccessUser(token, student.getUser(), student.getId());
            userService.createUserLogin(token, student.getUser());
            UserInfo userInfo = userService.getUserInfo(student.getUser(), token);
            userInfo.setStudentId(student.getId());
            userInfo.setIdentityNumber(student.getIdentityNumber());
            userInfo.setStudentCode(student.getStudentCode());
            return userInfo;
        }
    }

    public Student update(Student student) {
        student.setUpdateTime(new Date());
        User user = userRepo.findOne(student.getUser().getId());
        user.setName(student.getName());
        student.setUser(user);
        student = studentRepo.save(student);
        dataSendService.sendStudent(student);
        return student;
    }

}
